//+------------------------------------------------------------------+
//|                                      ChartObjectsTxtControls.mqh |
//|                        Copyright 2009, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
//|    .                                |
//+------------------------------------------------------------------+
#include "ChartObject.mqh"
//+------------------------------------------------------------------+
//|  CChartObjectText.                                          |
//+------------------------------------------------------------------+
//| :    "".                  |
//+------------------------------------------------------------------+
class CChartObjectText : public CChartObject
  {
public:
   //---     
   double            Angle();
   bool              Angle(double angle);
   string            Font();
   bool              Font(string font);
   int               FontSize();
   bool              FontSize(int size);
   int               Anchor();
   bool              Anchor(int anchor);
   //---   
   bool              Create(long chart_id,string name,int window,datetime time,double price);
   //---   
   virtual int       Type() { return(OBJ_TEXT); };
   //---    
   virtual bool      Save(int file_handle);
   virtual bool      Load(int file_handle);
  };
//+------------------------------------------------------------------+
//|   "".                                        |
//| INPUT:  chart_id- ,                          |
//|         name    - ,                                    |
//|         window  -  ,                          |
//|         time    - .                             |
//|         price   - .                                |
//| OUTPUT: true   ,  - false.                |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectText::Create(long chart_id,string name,int window,datetime time,double price)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_TEXT,window,time,price);
   if(result) result&=Attach(chart_id,name,window,1);
//---
   return(result);
  }
//+------------------------------------------------------------------+
//|   .                                          |
//| INPUT:  .                                                     |
//| OUTPUT:  .                                           |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
double CChartObjectText::Angle()
  {
//--- 
   if(m_chart_id==-1) return(0.0);
//---
   return(ObjectGetDouble(m_chart_id,m_name,OBJPROP_ANGLE));
  }
//+------------------------------------------------------------------+
//|   .                                        |
//| INPUT:  angle -  .                              |
//| OUTPUT: true-  ,  false.                        |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectText::Angle(double angle)
  {
//--- 
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetDouble(m_chart_id,m_name,OBJPROP_ANGLE,angle));
  }
//+------------------------------------------------------------------+
//|   .                                             |
//| INPUT:  .                                                     |
//| OUTPUT:  .                                              |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
string CChartObjectText::Font()
  {
//--- 
   if(m_chart_id==-1) return("");
//---
   return(ObjectGetString(m_chart_id,m_name,OBJPROP_FONT));
  }
//+------------------------------------------------------------------+
//|   .                                           |
//| INPUT:  font -  .                                  |
//| OUTPUT: true-  ,  false.                        |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectText::Font(string font)
  {
//--- 
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetString(m_chart_id,m_name,OBJPROP_FONT,font));
  }
//+------------------------------------------------------------------+
//|   .                                          |
//| INPUT:  .                                                     |
//| OUTPUT:  .                                           |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CChartObjectText::FontSize()
  {
//--- 
   if(m_chart_id==-1) return(0);
//---
   return((int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_FONTSIZE));
  }
//+------------------------------------------------------------------+
//|   .                                        |
//| INPUT:  size -  .                               |
//| OUTPUT: true-  ,  false.                        |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectText::FontSize(int size)
  {
//--- 
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_FONTSIZE,size));
  }
//+------------------------------------------------------------------+
//|   .                                          |
//| INPUT:  .                                                     |
//| OUTPUT:  .                                           |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CChartObjectText::Anchor()
  {
//--- 
   if(m_chart_id==-1) return(0);
//---
   return((int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_ANCHOR));
  }
//+------------------------------------------------------------------+
//|   .                                       |
//| INPUT:  anchor -  .                            |
//| OUTPUT: true-  ,  false.                        |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectText::Anchor(int anchor)
  {
//--- 
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_ANCHOR,anchor));
  }
//+------------------------------------------------------------------+
//|    .                                           |
//| INPUT:  file_handle -     .     |
//| OUTPUT: true  ,  false.                               |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectText::Save(int file_handle)
  {
   bool   resutl;
   int    len;
   string str;
//--- 
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//---  
   resutl=CChartObject::Save(file_handle);
   if(resutl)
     {
      //---   ""
      if(FileWriteDouble(file_handle,ObjectGetDouble(m_chart_id,m_name,OBJPROP_ANGLE))!=sizeof(double)) return(false);
      //---   ""
      str=ObjectGetString(m_chart_id,m_name,OBJPROP_FONT);
      len=StringLen(str);
      if(FileWriteInteger(file_handle,len,INT_VALUE)!=INT_VALUE) return(false);
      if(len!=0) if(FileWriteString(file_handle,str,len)!=len) return(false);
      //---   " "
      if(FileWriteInteger(file_handle,ObjectGetInteger(m_chart_id,m_name,OBJPROP_FONTSIZE),INT_VALUE)!=sizeof(int)) return(false);
      //---   " "
      if(FileWriteInteger(file_handle,ObjectGetInteger(m_chart_id,m_name,OBJPROP_ANCHOR),INT_VALUE)!=sizeof(int)) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//|    .                                         |
//| INPUT:  file_handle -     .     |
//| OUTPUT: true  ,  false.                               |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectText::Load(int file_handle)
  {
   bool   resutl;
   int    len;
   string str;
//--- 
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//---  
   resutl=CChartObject::Load(file_handle);
   if(resutl)
     {
      //---   ""
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_RAY_LEFT,FileReadInteger(file_handle,CHAR_VALUE))) return(false);
      //---   ""
      len=FileReadInteger(file_handle,INT_VALUE);
      if(len!=0) str=FileReadString(file_handle,len);
      else       str="";
      if(!ObjectSetString(m_chart_id,m_name,OBJPROP_FONT,str)) return(false);
      //---   " "
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_FONTSIZE,FileReadInteger(file_handle,INT_VALUE))) return(false);
      //---   " "
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_ANCHOR,FileReadInteger(file_handle,INT_VALUE))) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//|  CChartObjectLabel.                                         |
//+------------------------------------------------------------------+
//| :    "".                  |
//+------------------------------------------------------------------+
class CChartObjectLabel : public CChartObjectText
  {
public:
	void SetPos(int Corner=-1, int X=-1, int Y=-1, int dX=-1, int dY=-1);
   //---     
   int               X_Dinstance();
   bool              X_Dinstance(int X);
   int               Y_Dinstance();
   bool              Y_Dinstance(int Y);
   // ""
   datetime          Time() { return(0); };
   bool              Time(datetime time) {return(false); };
   double            Price() { return(0.0); };
   bool              Price(double price) {return(false); };
   //---   
   bool              Create(long chart_id,string name,int window,int X,int Y);
   //---   
   virtual int       Type() { return(OBJ_LABEL); };
   //---    
   virtual bool      Save(int file_handle);
   virtual bool      Load(int file_handle);
  };
//-------------------------------------------------------------- SetPos
void CChartObjectLabel::SetPos(int Corner=-1, int X=-1, int Y=-1, int dX=-1, int dY=-1)
{
	if (Corner>=0) ObjectSetInteger(m_chart_id, m_name, OBJPROP_CORNER, Corner);
	if (X>=0) ObjectSetInteger(m_chart_id, m_name, OBJPROP_XDISTANCE, X);
	if (Y>=0) ObjectSetInteger(m_chart_id, m_name, OBJPROP_YDISTANCE, Y);
	if (dX>=0) ObjectSetInteger(m_chart_id, m_name, OBJPROP_XSIZE, dX);
	if (dY>=0) ObjectSetInteger(m_chart_id, m_name, OBJPROP_YSIZE, dY);
}
//+------------------------------------------------------------------+
//|   "".                                        |
//| INPUT:  chart_id- ,                          |
//|         name    - ,                                    |
//|         window  -  ,                          |
//|         X       -         |
//|                   .                                |
//|         Y       -       |
//|                   .                                |
//| OUTPUT: true   ,  - false.                |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectLabel::Create(long chart_id,string name,int window,int X,int Y)
  {
   bool result=ObjectCreate(chart_id,name,OBJ_LABEL,window,0,0.0);
//---
   if(result) result&=Attach(chart_id,name,window,1);
   result&=Description(name);
   result&=X_Dinstance(X);
   result&=Y_Dinstance(Y);
//---
   return(result);
  }
//+------------------------------------------------------------------+
//|     X.                                     |
//| INPUT:  .                                                     |
//| OUTPUT:    X.                                      |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CChartObjectLabel::X_Dinstance()
  {
//--- 
   if(m_chart_id==-1) return(0);
//---
   return((int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_XDISTANCE));
  }
//+------------------------------------------------------------------+
//|     X.                                   |
//| INPUT:  X - .                                      |
//| OUTPUT: true-  ,  false.                        |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectLabel::X_Dinstance(int X)
  {
//--- 
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_XDISTANCE,X));
  }
//+------------------------------------------------------------------+
//|     Y.                                     |
//| INPUT:  .                                                     |
//| OUTPUT:    Y.                                      |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CChartObjectLabel::Y_Dinstance()
  {
//--- 
   if(m_chart_id==-1) return(0);
//---
   return((int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_YDISTANCE));
  }
//+------------------------------------------------------------------+
//|     Y.                                   |
//| INPUT:  Y - .                                      |
//| OUTPUT: true-  ,  false.                        |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectLabel::Y_Dinstance(int Y)
  {
//--- 
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_YDISTANCE,Y));
  }
//+------------------------------------------------------------------+
//|    .                                           |
//| INPUT:  file_handle -     .     |
//| OUTPUT: true  ,  false.                               |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectLabel::Save(int file_handle)
  {
   bool   resutl;
   int    len;
   string str;
//--- 
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//---  
   resutl=CChartObjectText::Save(file_handle);
   if(resutl)
     {
      //---   "   X"
      if(FileWriteInteger(file_handle,ObjectGetInteger(m_chart_id,m_name,OBJPROP_XDISTANCE),INT_VALUE)!=sizeof(int)) return(false);
      //---   "   Y"
      if(FileWriteInteger(file_handle,ObjectGetInteger(m_chart_id,m_name,OBJPROP_YDISTANCE),INT_VALUE)!=sizeof(int)) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//|    .                                         |
//| INPUT:  file_handle -     .     |
//| OUTPUT: true  ,  false.                               |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectLabel::Load(int file_handle)
  {
   bool   resutl;
   int    len;
   string str;
//--- 
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//---  
   resutl=CChartObjectText::Load(file_handle);
   if(resutl)
     {
      //---   "   X"
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_XDISTANCE,FileReadInteger(file_handle,INT_VALUE))) return(false);
      //---   "   Y"
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_YDISTANCE,FileReadInteger(file_handle,INT_VALUE))) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//|  CChartObjectEdit.                                          |
//+------------------------------------------------------------------+
//| :    " ".             |
//+------------------------------------------------------------------+
class CChartObjectEdit : public CChartObjectLabel
  {
public:
	bool Create(long chart_id, string name, int window, int Corner=0, int X=0, int Y=0, int dX=0, int dY=0, string Text="");
   //---     
   int               X_Size();
   bool              X_Size(int X);
   int               Y_Size();
   bool              Y_Size(int Y);
   color             BackColor();
   bool              BackColor(color new_color);
   // ""
   double            Angle()             { return(0.0); };
   bool              Angle(double angle) {return(false); };
   //---   
   bool              Create(long chart_id,string name,int window,int X,int Y,int sizeX,int sizeY);
   //---   
   virtual int       Type() { return(OBJ_EDIT); };
   //---    
   virtual bool      Save(int file_handle);
   virtual bool      Load(int file_handle);
  };
//-------------------------------------------------------------- Create
bool CChartObjectEdit::Create(long chart_id, string name, int window, int Corner=0, int X=0, int Y=0, int dX=0, int dY=0, string Text="")
{
	bool result=ObjectCreate(chart_id,name, (ENUM_OBJECT)Type(), window, 0, 0, 0, 0, 0);
	if(result) result&=Attach(chart_id,name,window,1);
//---
	if (result)
	{
		ObjectSetInteger(chart_id, name, OBJPROP_CORNER, Corner);
		ObjectSetInteger(chart_id, name, OBJPROP_XDISTANCE, X);
		ObjectSetInteger(chart_id, name, OBJPROP_YDISTANCE, Y);
		ObjectSetInteger(chart_id, name, OBJPROP_XSIZE, dX);
		ObjectSetInteger(chart_id, name, OBJPROP_YSIZE, dY);
		ObjectSetString(chart_id, name, OBJPROP_TEXT, Text);
		ObjectSetString(chart_id, name, OBJPROP_FONT, "Tahoma");
		ObjectSetInteger(chart_id, name, OBJPROP_FONTSIZE, 9);
		ObjectSetInteger(chart_id, name, OBJPROP_STATE, 0);
		ObjectSetInteger(chart_id, name, OBJPROP_SELECTABLE, 0);
	}
	return(result);
}
//+------------------------------------------------------------------+
//|   " ".                                   |
//| INPUT:  chart_id- ,                          |
//|         name    - ,                                    |
//|         window  -  ,                          |
//|         X       -         |
//|                   ,                                |
//|         Y       -       |
//|                   ,                                |
//|         sizeX   -   X,                                |
//|         sizeY   -   Y.                                |
//| OUTPUT: true   ,  - false.                |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectEdit::Create(long chart_id,string name,int window,int X,int Y,int sizeX,int sizeY)
  {
   bool result=ObjectCreate(chart_id,name,(ENUM_OBJECT)Type(),window,0,0,0);
   if(result) result&=Attach(chart_id,name,window,1);
   result&=X_Dinstance(X);
   result&=Y_Dinstance(Y);
   result&=X_Size(sizeX);
   result&=Y_Size(sizeY);
//---
   return(result);
  }
//+------------------------------------------------------------------+
//|     X.                                        |
//| INPUT:  .                                                     |
//| OUTPUT:    X.                                         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CChartObjectEdit::X_Size()
  {
//--- 
   if(m_chart_id==-1) return(0);
//---
   return((int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_XSIZE));
  }
//+------------------------------------------------------------------+
//|     X.                                      |
//| INPUT:  X - .                                         |
//| OUTPUT: true-  ,  false.                        |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectEdit::X_Size(int X)
  {
//--- 
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_XSIZE,X));
  }
//+------------------------------------------------------------------+
//|     Y.                                        |
//| INPUT:  .                                                     |
//| OUTPUT:    Y.                                         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CChartObjectEdit::Y_Size()
  {
//--- 
   if(m_chart_id==-1) return(0);
//---
   return((int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_YSIZE));
  }
//+------------------------------------------------------------------+
//|     Y.                                      |
//| INPUT:  Y - .                                         |
//| OUTPUT: true-  ,  false.                        |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectEdit::Y_Size(int Y)
  {
//--- 
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_YSIZE,Y));
  }
//+------------------------------------------------------------------+
//|   .                                              |
//| INPUT:  .                                                     |
//| OUTPUT:  .                                               |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
color CChartObjectEdit::BackColor()
  {
//--- 
   if(m_chart_id==-1) return(0);
//---
   return((int)ObjectGetInteger(m_chart_id,m_name,OBJPROP_BGCOLOR));
  }
//+------------------------------------------------------------------+
//|   .                                            |
//| INPUT:  new_color -  .                              |
//| OUTPUT: true-  ,  false.                        |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectEdit::BackColor(color new_color)
  {
//--- 
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_BGCOLOR,new_color));
  }
//+------------------------------------------------------------------+
//|    .                                           |
//| INPUT:  file_handle -     .     |
//| OUTPUT: true  ,  false.                               |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectEdit::Save(int file_handle)
  {
   bool   resutl;
   int    len;
   string str;
//--- 
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//---  
   resutl=CChartObjectLabel::Save(file_handle);
   if(resutl)
     {
      //---   "   X"
      if(FileWriteInteger(file_handle,ObjectGetInteger(m_chart_id,m_name,OBJPROP_XSIZE),INT_VALUE)!=sizeof(int)) return(false);
      //---   "   Y"
      if(FileWriteInteger(file_handle,ObjectGetInteger(m_chart_id,m_name,OBJPROP_YSIZE),INT_VALUE)!=sizeof(int)) return(false);
      //---   
      if(FileWriteLong(file_handle,ObjectGetInteger(m_chart_id,m_name,OBJPROP_BGCOLOR))!=sizeof(long)) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//|    .                                         |
//| INPUT:  file_handle -     .     |
//| OUTPUT: true  ,  false.                               |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectEdit::Load(int file_handle)
  {
   bool   resutl;
   int    len;
   string str;
//--- 
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//---  
   resutl=CChartObjectLabel::Load(file_handle);
   if(resutl)
     {
      //---   "   X"
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_XSIZE,FileReadInteger(file_handle,INT_VALUE))) return(false);
      //---   "   Y"
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_YSIZE,FileReadInteger(file_handle,INT_VALUE))) return(false);
      //---   
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_BGCOLOR,FileReadLong(file_handle))) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//|  CChartObjectButton.                                        |
//+------------------------------------------------------------------+
//| :    "".                 |
//+------------------------------------------------------------------+
class CChartObjectButton : public CChartObjectEdit
  {
public:
   //---     
   bool              State();
   bool              State(bool state);
   //---   
   virtual int       Type() { return(OBJ_BUTTON); };
   //---    
   virtual bool      Save(int file_handle);
   virtual bool      Load(int file_handle);
  };
//+------------------------------------------------------------------+
//|   .                                       |
//| INPUT:  .                                                     |
//| OUTPUT:  .                                        |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectButton::State()
  {
//--- 
   if(m_chart_id==-1) return(0.0);
//---
   return(ObjectGetInteger(m_chart_id,m_name,OBJPROP_STATE));
  }
//+------------------------------------------------------------------+
//|   .                                     |
//| INPUT:  state -  .                           |
//| OUTPUT: true-  ,  false.                        |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectButton::State(bool state)
  {
//--- 
   if(m_chart_id==-1) return(false);
//---
   return(ObjectSetInteger(m_chart_id,m_name,OBJPROP_STATE,state));
  }
//+------------------------------------------------------------------+
//|    .                                           |
//| INPUT:  file_handle -     .     |
//| OUTPUT: true  ,  false.                               |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectButton::Save(int file_handle)
  {
   bool   resutl;
   int    len;
   string str;
//--- 
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//---  
   resutl=CChartObjectEdit::Save(file_handle);
   if(resutl)
     {
      //---   
      if(FileWriteLong(file_handle,ObjectGetInteger(m_chart_id,m_name,OBJPROP_STATE))!=sizeof(long)) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
//|    .                                         |
//| INPUT:  file_handle -     .     |
//| OUTPUT: true  ,  false.                               |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CChartObjectButton::Load(int file_handle)
  {
   bool   resutl;
   int    len;
   string str;
//--- 
   if(file_handle<=0) return(false);
   if(m_chart_id==-1) return(false);
//---  
   resutl=CChartObjectEdit::Load(file_handle);
   if(resutl)
     {
      //---   
      if(!ObjectSetInteger(m_chart_id,m_name,OBJPROP_STATE,FileReadLong(file_handle))) return(false);
     }
//---
   return(resutl);
  }
//+------------------------------------------------------------------+
